#!/usr/bin/env python3

import sys
from pathlib import Path


# Get command-line arguments
page = Path(sys.argv[1])

pagename = page.parent
if pagename != Path("."):
    print(f": {str(pagename)}", end="")



#!/usr/bin/env python3
#
# © Reuben Thomas <rrt@sc3d.org> 2024-2026
# Released under the GPL version 3, or (at your option) any later version.

import os
import sys
import urllib.parse
from pathlib import Path


# Directory listing generator
def make_directory(
    path: Path, url: str, link_classes: str, dir_link_classes: str
) -> str:
    entries: list[Path] = []
    for entry in path.iterdir():
        if entry.is_dir() and os.access(entry, os.R_OK):
            entries.append(entry)
    pages = ""
    dirs = ""
    for entry in sorted(entries):
        quoted_entry = urllib.parse.quote(entry.name)
        link = f'<a href="$run(path-to-root.in.py,$path)/{url}{quoted_entry}/index.html">{entry.name}</a>'
        if any(subentry.is_dir() for subentry in entry.iterdir()):
            dirs += f'<li><span class="{dir_link_classes}">{link}</span></li>'
        else:
            pages += f'<li><span class="{link_classes}">{link}</span></li>'
    return dirs + pages


# Read command-line arguments
def maybe_argv(n: int) -> str | None:
    return sys.argv[n] if len(sys.argv) > n else None


page = Path(sys.argv[1])
realpath = Path(os.environ["NANCY_INPUT"]) / page
directory_str = maybe_argv(2)
directory = Path(directory_str) if directory_str is not None else page.parent
link_classes = maybe_argv(3) or "nav-link"
dir_link_classes = maybe_argv(4) or "nav-link nav-directory"

path = directory.parent
if directory == Path("."):
    parent_directory = realpath.parent
else:
    parent_directory = realpath.parent.parent

url = urllib.parse.quote(str(path))
if url == ".":
    url = ""
else:
    url += "/"
print(make_directory(parent_directory, url, link_classes, dir_link_classes))



"""Linton tests utility routines.

Copyright (c) Reuben Thomas 2023-2026.
Released under the GPL version 3, or (at your option) any later version.
"""

import contextlib
import filecmp
import io
import os
import re
import shutil
import sys
import tempfile
from collections.abc import Callable
from dataclasses import dataclass
from pathlib import Path
from typing import Any
from unittest.mock import patch

import pytest
from pytest import CaptureFixture, mark, param


@dataclass
class Case:
    name: str
    args: list[str]
    error: int | None = None


# See https://stackoverflow.com/questions/4187564
def dirs_equal(a: Path, b: Path) -> bool:
    stdout = io.StringIO()
    with contextlib.redirect_stdout(stdout):
        filecmp.dircmp(a, b).report_full_closure()
    match = re.search("Differing files|Only in", stdout.getvalue())
    if match is None:
        return True
    print(stdout.getvalue())
    return False


def dir_test(
    function: Callable[[list[str]], None],
    case: Case,
    fixture_dir: Path,
    capsys: CaptureFixture[str],
    regenerate_expected: bool,
) -> None:
    subcommand_name = case.args[0]
    expected_dir = fixture_dir / subcommand_name / case.name / "expected"
    expected_stderr = fixture_dir / subcommand_name / case.name / "expected-stderr.txt"
    patched_argv = [subcommand_name, *(sys.argv[1:])]
    with tempfile.TemporaryDirectory(delete="DEBUG" not in os.environ) as tmpdir:
        output_dir = Path(tmpdir) / "output"
        full_args = [*case.args, str(output_dir)]
        if case.error is None:
            with patch("sys.argv", patched_argv):
                function(full_args)
            if regenerate_expected:
                shutil.copytree(output_dir, expected_dir, dirs_exist_ok=True)
            else:
                assert dirs_equal(output_dir, expected_dir)
        else:
            with pytest.raises(SystemExit) as e:
                with patch("sys.argv", patched_argv):
                    function(full_args)
            assert e.value.code == case.error
        if regenerate_expected:
            with open(expected_stderr, "w", encoding="utf-8") as f:
                f.write(capsys.readouterr().err)
        else:
            assert (
                capsys.readouterr().err
                == open(expected_stderr, encoding="utf-8").read()
            )


def make_tests(
    function: Callable[..., Any],
    fixture_dir: Path,
    *tests: Case,
) -> Any:
    ids = []
    test_cases = []
    for t in tests:
        ids.append(t.name)
        test_cases.append(t)
    return mark.parametrize(
        "function,case,fixture_dir",
        [param(function, case, fixture_dir) for case in test_cases],
        ids=ids,
    )



#!/usr/bin/env python3

import os
import sys
from datetime import datetime
from pathlib import Path


# Get command-line arguments
basename = sys.argv[1]
file = Path(os.environ["NANCY_INPUT"]).parent / basename

time = os.stat(file).st_mtime
dt = datetime.fromtimestamp(time)
print(dt.strftime("%Y/%m/%d"), end="")



#!/usr/bin/env python3

import sys
import urllib.parse
from pathlib import Path


# Get command-line arguments
page = Path(sys.argv[1])

# Extract page name and normalize path
path = page.parent

# Generate and print breadcrumb
tree = ""
classes = "breadcrumb-item breadcrumb-active"
while str(path) not in (".", "/"):
    quoted_parents = urllib.parse.quote(str(path))
    tree = (
        f'<li class="{classes}">'
        + f'<a href="$run(path-to-root.in.py,$path)/{quoted_parents}/index.html">{path.name}</a>'
        + f"</li>{tree}"
    )
    classes = "breadcrumb-item"
    path = path.parent
print(
    '<li class="breadcrumb-item">'
    + '<a href="$run(path-to-root.in.py,$path)/index.html">$include(Title.in.txt)</a>'
    + f"</li>{tree}"
)



"""Linton 'serve' subcommand

© Reuben Thomas <rrt@sc3d.org> 2024-2025
Released under the GPL version 3, or (at your option) any later version.
"""

import argparse
import glob
import os
import subprocess
import urllib.parse
from http.server import BaseHTTPRequestHandler, HTTPServer
from mimetypes import guess_type
from pathlib import Path
from tempfile import TemporaryDirectory
from urllib.parse import quote


def denancify(name: Path) -> Path:
    suffixes = name.suffixes
    if len(suffixes) > 0 and ".nancy" in suffixes[-2:]:
        suffixes.remove(".nancy")
        while name.suffix:
            name = name.with_suffix("")
        return name.with_suffix("".join(suffixes))
    return name


def run(args: argparse.Namespace) -> None:
    """'serve' command handler"""

    class HTTPRequestHandler(BaseHTTPRequestHandler):
        def serve_file(self, filename: Path, content: bytes) -> None:
            mime_type = guess_type(filename)[0] or "text/plain"
            self.send_response(200)
            self.send_header("Content-Type", mime_type)
            self.end_headers()
            self.wfile.write(content)

        def maybe_serve_file(
            self, filename: Path, filenames: list[str], url_path: str
        ) -> bool:
            # If the name is a directory, try serving its "index.html"
            if filename.is_dir():
                self.send_response(301)
                self.send_header(
                    "Location",
                    os.path.join(quote(args.base_url), quote(url_path), "index.html"),
                )
                self.end_headers()
                return True

            # First, try reading a plain file.
            if filename.name in filenames:
                with open(filename, "rb") as fh:
                    output = fh.read()
                self.serve_file(filename, output)
                return True

            # Next, look for a Nancy source file to expand.
            suffix = filename.suffix
            nancy_suffix = f".nancy{suffix}"
            nancy_source = filename.with_suffix(nancy_suffix)
            if nancy_source.name in filenames:
                output = self.run_command(
                    [
                        "nancy",
                        args.document_root,
                        "-",
                        f"--path={Path(url_path).parent / nancy_source.name}",
                    ],
                )
                if output is None:
                    return True
                self.serve_file(Path(nancy_source), output)
                return True

            return False

        def run_command(self, cmd: list[str]) -> bytes | None:
            try:
                res = subprocess.run(cmd, capture_output=True, check=True)
                return res.stdout
            except subprocess.CalledProcessError as e:
                self.send_response(500)
                self.send_header("Content-Type", "text/html")
                self.end_headers()
                self.wfile.write(
                    b"<html><head><title>Internal error</title><body><h1>Internal error</h1><pre>"
                    + e.stderr
                    + b"</pre></body></html>"
                )

        def do_GET(self) -> None:
            """GET handler"""
            url_path = urllib.parse.unquote(
                urllib.parse.urlparse(self.path.removeprefix(args.base_url)).path
            )

            # First, try the literal file name and its templated version.
            input_path = Path(args.document_root) / url_path
            if self.maybe_serve_file(
                input_path, os.listdir(input_path.parent), url_path
            ):
                return

            # If that doesn't work, find all files whose names containing
            # commands in the same directory, and expand a file containing
            # each of them to find its expanded name, in case one expands to
            # the name we want.
            with TemporaryDirectory() as tmpdir:
                filename_file_url = Path(os.path.dirname(url_path)) / "filelist.nancy"
                filename_file = Path(tmpdir) / filename_file_url
                os.makedirs(filename_file.parent, exist_ok=True)
                for filename in glob.glob(b"*$*", root_dir=bytes(input_path.parent)):
                    with open(filename_file, "wb") as f:
                        f.write(filename)
                    output_file = self.run_command(
                        [
                            "nancy",
                            os.pathsep.join([args.document_root, tmpdir]),
                            "-",
                            f"--path={filename_file_url}",
                        ],
                    )
                    if output_file is None:
                        return
                    output_name = denancify(Path(output_file.decode("utf-8")))
                    if input_path.name == str(output_name) and self.maybe_serve_file(
                        input_path.parent / denancify(Path(filename.decode("utf-8"))),
                        [os.fsdecode(filename)],
                        url_path,
                    ):
                        return

            # Otherwise, file is not found
            self.send_response(404)
            self.send_header("Content-Type", "text/html")
            self.end_headers()
            self.wfile.write(
                b"<html><head><title>No such page</title><body>No such page</body></html>"
            )

    httpd = HTTPServer(("localhost", args.port), HTTPRequestHandler)
    (host, port, *_) = httpd.server_address
    print(f"Connect to server at http://{str(host)}:{port}/index.html")
    httpd.serve_forever()


def add_subparser(subparsers: argparse._SubParsersAction) -> None:
    parser = subparsers.add_parser(
        "serve",
        help="serve a Linton web site locally on your computer, for testing",
    )
    parser.add_argument(
        "-p",
        "--port",
        help="port on which to listen [default: random]",
        type=int,
        default=0,
    )
    parser.add_argument(
        "document_root",
        metavar="DIRECTORY",
        help="directory containing source files [default: current working directory]",
        default=os.getcwd(),
        nargs="?",
    )
    parser.set_defaults(func=run)
